local L = LibStub("AceLocale-3.0"):NewLocale("GridStatusHots", "enUS", true)
L["Banzai alert"] = true
L["Banzai incoming spell"] = true

local L = LibStub("AceLocale-3.0"):NewLocale("GridStatusBanzai", "zhCN")
if L then
	L["Banzai alert"] = "怪物目标"
	L["Banzai incoming spell"] = "怪物施法目标"
end

local L = LibStub("AceLocale-3.0"):NewLocale("GridStatusBanzai", "zhTW")
if L then
	L["Banzai alert"] = "怪物目標"
	L["Banzai incoming spell"] = "怪物施法目標"
end

local L = LibStub("AceLocale-3.0"):NewLocale("GridStatusBanzai", "ruRU")
if L then
	L["Banzai alert"] = "Цель врага"
	L["Banzai incoming spell"] = "Цель заклинания"
end

local L = LibStub("AceLocale-3.0"):GetLocale("GridStatusBanzai")


local GridRoster = Grid:GetModule("GridRoster")

local GridStatusBanzai = Grid:GetModule("GridStatus"):NewModule("GridStatusBanzai", "AceTimer-3.0")
GridStatusBanzai.menuName = L["Banzai alert"]

--{{{ AceDB defaults

local UpdateInterval = 0.1

GridStatusBanzai.defaultDB = {
	debug = false,
	alert_banzai = {
		text =  "!!",
		enable = true,
		color = { r = 1, g = 1, b = 0, a = 1 },
		priority = 98,
		range = false,
	},
	alert_banzai_incoming = {
		text =  "->",
		enable = true,
		color = { r = 1, g = 1, b = 0, a = 1 },
		priority = 98,
		range = false,
	},
}

--}}}

function GridStatusBanzai:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("alert_banzai", L["Banzai alert"], nil, true)
	self:RegisterStatus("alert_banzai_incoming", L["Banzai incoming spell"], nil, true)
end

local spellstarts = {}      --save spell casting event, [CastingNpcGUID] -> { spellnames, channeling }.
local incomings =   {}      --save incomings on raid members, [RosterGUID] -> { t = timeLeft, m = CastingNpcGUID }
local castings =    {}      --revert map of incomings, always match, [CastingNpcGUID] -> RosterGUID

function GridStatusBanzai:IsHostileNpcUnit(guid)
	return not GridRoster:IsGUIDInRaid(guid)
end

function GridStatusBanzai:COMBAT_LOG_EVENT_UNFILTERED(b, timestamp, event, sid, sname, sflag, tid, tname, tflag, spellid)
	if event=="SPELL_CAST_START" or event=="SPELL_CAST_SUCCESS" then
		if self:IsHostileNpcUnit(sid) then
			spellstarts[sid] = { GetSpellInfo(spellid), event=="SPELL_CAST_SUCCESS" } --save spell name for next checking target of the caster GUID
		end
	elseif event=="SPELL_INTERRUPT" then
		self:TryToStop(tid)
	elseif event=="UNIT_DIED" then
		self:TryToStop(tid)
	end
end

function GridStatusBanzai:TryToStop(npcguid)
	if(castings[npcguid]) then
		GridStatusBanzai.core:SendStatusLost(castings[npcguid],"alert_banzai_incoming")
		incomings[castings[npcguid]] = nil
		castings[npcguid]=nil
	end
end

function GridStatusBanzai:UnitCastStop(unitid, name, rank)
	self:TryToStop(UnitGUID(unitid))
end

function GridStatusBanzai:ResetVariables()
	spellstarts = {}
	for guid, v in pairs(castings) do
		self:TryToStop(guid)
	end
end

local trackIncoming
local timer

function GridStatusBanzai:OnStatusEnable(status)
	if (status=="alert_banzai" or status=="alert_banzai_incoming") then
		if timer then 
			self:CancelTimer(timer)
			timer = nil
		end
		timer = self:ScheduleRepeatingTimer("OnUpdate", UpdateInterval)
	end
	if (status=="alert_banzai") then
		trackIncoming = true
	end
	if (status=="alert_banzai_incoming") then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "UnitCastStop") --caster stop manually, only for players.
		self:RegisterEvent("UNIT_SPELLCAST_STOP", "UnitCastStop")
		self:RegisterEvent("PLAYER_REGEN_DISABLED", "ResetVariables");
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "ResetVariables");
	end
end

function GridStatusBanzai:OnStatusDisable(status)
	if (status=="alert_banzai" or status=="alert_banzai_incoming") then
		if (not (GridStatusBanzai.db.profile["alert_banzai"].enable or GridStatusBanzai.db.profile["alert_banzai_incoming"].enable)) then
			if timer then 
				self:CancelTimer(timer)
				timer = nil
			end
		end
	end
	if (status=="alert_banzai") then
		trackIncoming = false
	end
	if (status=="alert_banzai_incoming") then
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP") --caster stop manually, only for players.
		self:UnregisterEvent("UNIT_SPELLCAST_STOP")
		self:UnregisterEvent("PLAYER_REGEN_DISABLED");
		self:UnregisterEvent("PLAYER_REGEN_ENABLED");
	end
end

local newcount = {}

--cache for string concat, learn from LibBanzai
local targets = setmetatable({}, {__index = function(self, key) self[key] = key .. "target" return self[key] end})

function GridStatusBanzai:OnUpdate()
	for guid, v in pairs(incomings) do
		v.time = v.time-UpdateInterval
		if(v.time<=0)then
			GridStatusBanzai.core:SendStatusLost(guid,"alert_banzai_incoming")
			castings[incomings[guid].npc]=nil
			incomings[guid]=nil
		end
	end

	table.wipe(newcount)

	self:UpdateUnit("focus")
	self:UpdateUnit("mouseover")
	for guid, unitid in GridRoster:IterateRoster() do
		self:UpdateUnit(targets[unitid])
	end

	if (trackIncoming) then
		for guid, unitid in GridRoster:IterateRoster() do
			if(newcount[guid]) then
				local settings = GridStatusBanzai.db.profile.alert_banzai
				GridStatusBanzai.core:SendStatusGained(
					guid,
					"alert_banzai",
					settings.priority,
					(settings.range and 40),
					settings.color,
					settings.text,
					nil,
					nil,
					settings.icon
				)
			else
				GridStatusBanzai.core:SendStatusLost(guid, "alert_banzai")
			end
		end
	end
end

function GridStatusBanzai:UpdateUnit(npcunit)
	local spell, _, _, icon, startTime, endTime
	local npcguid = UnitGUID(npcunit)
	if( npcguid and self:IsHostileNpcUnit(npcguid) ) then
		local guid = UnitGUID(targets[npcunit])
		if guid and GridRoster:IsGUIDInRaid(guid) then
			newcount[guid]=true

			local starts = spellstarts[npcguid]
			if(starts) then
				spellstarts[npcguid]=nil
				if(starts[2]) then
					spell, _, _, icon, startTime, endTime = UnitChannelInfo(npcunit)
				else
					spell, _, _, icon, startTime, endTime = UnitCastingInfo(npcunit)
				end
				if(spell and spell==starts[1]) then
					incomings[guid]={ 
						time = endTime/1000-GetTime(), 
						npc = npcguid 
					}
					castings[npcguid] = guid

					local settings = GridStatusBanzai.db.profile.alert_banzai_incoming
					GridStatusBanzai.core:SendStatusGained(
						guid,
						"alert_banzai_incoming",
						settings.priority,
						(settings.range and 40),
						settings.color,
						settings.text,
						nil,
						nil,
						icon,
						startTime/1000,
						(endTime-startTime)/1000
					)
				end
			end
		end
	end
end